==> datafusion-0.8.0.tar.gz: datafusion-0.8.0/src/expr/projection.rs <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

use datafusion_expr::logical_plan::Projection;
use pyo3::prelude::*;
use std::fmt::{self, Display, Formatter};

use crate::common::df_schema::PyDFSchema;
use crate::expr::logical_node::LogicalNode;
use crate::expr::PyExpr;
use crate::sql::logical::PyLogicalPlan;

#[pyclass(name = "Projection", module = "datafusion.expr", subclass)]
#[derive(Clone)]
pub struct PyProjection {
    projection: Projection,
}

impl PyProjection {
    pub fn new(projection: Projection) -> Self {
        Self { projection }
    }
}

impl From<Projection> for PyProjection {
    fn from(projection: Projection) -> PyProjection {
        PyProjection { projection }
    }
}

impl From<PyProjection> for Projection {
    fn from(proj: PyProjection) -> Self {
        proj.projection
    }
}

impl Display for PyProjection {
    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
        write!(
            f,
            "Projection
            \nExpr(s): {:?}
            \nInput: {:?}
            \nProjected Schema: {:?}",
            &self.projection.expr, &self.projection.input, &self.projection.schema,
        )
    }
}

#[pymethods]
impl PyProjection {
    /// Retrieves the expressions for this `Projection`
    fn projections(&self) -> PyResult<Vec<PyExpr>> {
        Ok(self
            .projection
            .expr
            .iter()
            .map(|e| PyExpr::from(e.clone()))
            .collect())
    }

    /// Retrieves the input `LogicalPlan` to this `Projection` node
    fn input(&self) -> PyLogicalPlan {
        PyLogicalPlan::from((*self.projection.input).clone())
    }

    /// Resulting Schema for this `Projection` node instance
    fn schema(&self) -> PyResult<PyDFSchema> {
        Ok((*self.projection.schema).clone().into())
    }

    fn __repr__(&self) -> PyResult<String> {
        Ok(format!("Projection({})", self))
    }
}

impl LogicalNode for PyProjection {
    fn input(&self) -> Vec<PyLogicalPlan> {
        vec![PyLogicalPlan::from((*self.projection.input).clone())]
    }
}

==> datafusion-23.0.0.tar.gz: datafusion-23.0.0/src/expr/projection.rs <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

use datafusion_expr::logical_plan::Projection;
use pyo3::prelude::*;
use std::fmt::{self, Display, Formatter};

use crate::common::df_schema::PyDFSchema;
use crate::expr::logical_node::LogicalNode;
use crate::expr::PyExpr;
use crate::sql::logical::PyLogicalPlan;

#[pyclass(name = "Projection", module = "datafusion.expr", subclass)]
#[derive(Clone)]
pub struct PyProjection {
    projection: Projection,
}

impl PyProjection {
    pub fn new(projection: Projection) -> Self {
        Self { projection }
    }
}

impl From<Projection> for PyProjection {
    fn from(projection: Projection) -> PyProjection {
        PyProjection { projection }
    }
}

impl From<PyProjection> for Projection {
    fn from(proj: PyProjection) -> Self {
        proj.projection
    }
}

impl Display for PyProjection {
    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
        write!(
            f,
            "Projection
            \nExpr(s): {:?}
            \nInput: {:?}
            \nProjected Schema: {:?}",
            &self.projection.expr, &self.projection.input, &self.projection.schema,
        )
    }
}

#[pymethods]
impl PyProjection {
    /// Retrieves the expressions for this `Projection`
    fn projections(&self) -> PyResult<Vec<PyExpr>> {
        Ok(self
            .projection
            .expr
            .iter()
            .map(|e| PyExpr::from(e.clone()))
            .collect())
    }

    /// Retrieves the input `LogicalPlan` to this `Projection` node
    fn input(&self) -> PyResult<Vec<PyLogicalPlan>> {
        Ok(Self::inputs(self))
    }

    /// Resulting Schema for this `Projection` node instance
    fn schema(&self) -> PyResult<PyDFSchema> {
        Ok((*self.projection.schema).clone().into())
    }

    fn __repr__(&self) -> PyResult<String> {
        Ok(format!("Projection({})", self))
    }

    fn __name__(&self) -> PyResult<String> {
        Ok("Projection".to_string())
    }
}

impl LogicalNode for PyProjection {
    fn inputs(&self) -> Vec<PyLogicalPlan> {
        vec![PyLogicalPlan::from((*self.projection.input).clone())]
    }

    fn to_variant(&self, py: Python) -> PyResult<PyObject> {
        Ok(self.clone().into_py(py))
    }
}
